@@ -57,7 +57,6 @@ class ExpressionClassBody(MarkLocalsDictIndicatorMixin,
             self,
             provider    = provider,
             name        = name,
-            is_class    = True,
             code_prefix = "class",
             flags       = flags,
             source_ref  = source_ref
@@ -69,8 +68,6 @@ class ExpressionClassBody(MarkLocalsDictIndicatorMixin,
 
         self.doc = doc
 
-        assert self.isEarlyClosure()
-
         self.has_annotations = False
 
     def getDetail(self):
@@ -110,6 +107,10 @@ class ExpressionClassBody(MarkLocalsDictIndicatorMixin,
     def getDoc(self):
         return self.doc
 
+    @staticmethod
+    def isEarlyClosure():
+        return True
+
     def getVariableForClosure(self, variable_name):
         # print( "getVariableForClosure", self, variable_name )
 
