@@ -116,7 +116,6 @@ class ExpressionCoroutineObjectBody(MarkLocalsDictIndicatorMixin, ExpressionFunc
             provider    = provider,
             name        = name,
             code_prefix = "coroutine",
-            is_class    = False,
             flags       = flags,
             source_ref  = source_ref
         )
@@ -128,9 +127,6 @@ class ExpressionCoroutineObjectBody(MarkLocalsDictIndicatorMixin, ExpressionFunc
     def getFunctionName(self):
         return self.name
 
-    def getFunctionQualname(self):
-        return self.getParentVariableProvider().getFunctionQualname()
-
     def markAsNeedsGeneratorReturnHandling(self, value):
         self.needs_generator_return_exit = max(
             self.needs_generator_return_exit,
