@@ -31,7 +31,6 @@ from nuitka.importing.Importing import (
 )
 from nuitka.importing.Recursion import decideRecursion, recurseTo
 from nuitka.ModuleRegistry import getOwnerFromCodeName
-from nuitka.nodes.IndicatorMixins import MarkNeedsAnnotationsMixin
 from nuitka.optimizations.TraceCollections import TraceCollectionModule
 from nuitka.PythonVersions import python_version
 from nuitka.SourceCodeReferences import SourceCodeReference, fromFilename
@@ -41,7 +40,8 @@ from nuitka.utils.FileOperations import relpath
 from .Checkers import checkStatementsSequenceOrNone
 from .ConstantRefNodes import makeConstantRefNode
 from .ExpressionBases import ExpressionBase
-from .FutureSpecs import FutureSpec
+from .FutureSpecs import FutureSpec, fromFlags
+from .IndicatorMixins import MarkNeedsAnnotationsMixin
 from .NodeBases import (
     ChildrenHavingMixin,
     ClosureGiverNodeMixin,
@@ -156,6 +156,7 @@ class PythonModuleBase(NodeBase):
         return os.path.abspath(self.getSourceReference().getFilename())
 
     def getRunTimeFilename(self):
+        # TODO: Don't look at such things this late, push this into building.
         reference_mode = Options.getFileReferenceMode()
 
         if reference_mode == "original":
@@ -202,7 +203,7 @@ class CompiledPythonModule(ChildrenHavingMixin, ClosureGiverNodeMixin,
         "body": checkStatementsSequenceOrNone
     }
 
-    def __init__(self, name, package_name, mode, source_ref):
+    def __init__(self, name, package_name, mode, future_spec, source_ref):
         PythonModuleBase.__init__(
             self,
             name         = name,
@@ -236,7 +237,8 @@ class CompiledPythonModule(ChildrenHavingMixin, ClosureGiverNodeMixin,
         # SSA trace based information about the module.
         self.trace_collection = None
 
-        self.future_spec = None
+        # Often None until tree building finishes its part.
+        self.future_spec = future_spec
 
     def getDetails(self):
         return {
@@ -248,7 +250,8 @@ class CompiledPythonModule(ChildrenHavingMixin, ClosureGiverNodeMixin,
     def getDetailsForDisplay(self):
         result = self.getDetails()
 
-        result["code_flags"] = ','.join(self.future_spec.asFlags())
+        if self.future_spec is not None:
+            result["code_flags"] = ','.join(self.future_spec.asFlags())
 
         return result
 
@@ -261,7 +264,7 @@ class CompiledPythonModule(ChildrenHavingMixin, ClosureGiverNodeMixin,
         return self.future_spec
 
     def asGraph(self, computation_counter):
-        from graphviz import Digraph # @UnresolvedImport pylint: disable=import-error,useless-suppression
+        from graphviz import Digraph # @UnresolvedImport pylint: disable=I0021,import-error
 
         graph = Digraph("cluster_%d" % computation_counter, comment = "Graph for %s" % self.getName())
         graph.body.append("style=filled")
@@ -356,9 +359,9 @@ class CompiledPythonModule(ChildrenHavingMixin, ClosureGiverNodeMixin,
     def getContainingClassDictCreation():
         return None
 
-    def isEarlyClosure(self):
+    @staticmethod
+    def isEarlyClosure():
         # Modules should immediately closure variables on use.
-        # pylint: disable=no-self-use
         return True
 
     def getCodeName(self):
@@ -420,12 +423,6 @@ class CompiledPythonModule(ChildrenHavingMixin, ClosureGiverNodeMixin,
         # Scons or unknown, so lets avoid them for now.
         return result.replace(')',"").replace('(',"")
 
-    # TODO: Can't really use locals for modules, this should probably be made
-    # sure to not be used.
-    @staticmethod
-    def getLocalsMode():
-        return "copy"
-
     def computeModule(self):
         old_collection = self.trace_collection
 
@@ -462,7 +459,7 @@ class CompiledPythonModule(ChildrenHavingMixin, ClosureGiverNodeMixin,
 class CompiledPythonPackage(CompiledPythonModule):
     kind = "COMPILED_PYTHON_PACKAGE"
 
-    def __init__(self, name, package_name, mode, source_ref):
+    def __init__(self, name, package_name, mode, future_spec, source_ref):
         assert name
 
         CompiledPythonModule.__init__(
@@ -470,6 +467,7 @@ class CompiledPythonPackage(CompiledPythonModule):
             name         = name,
             package_name = package_name,
             mode         = mode,
+            future_spec  = future_spec,
             source_ref   = source_ref
         )
 
@@ -567,12 +565,13 @@ class UncompiledPythonPackage(UncompiledPythonModule):
 class PythonMainModule(CompiledPythonModule):
     kind = "PYTHON_MAIN_MODULE"
 
-    def __init__(self, main_added, mode, source_ref):
+    def __init__(self, main_added, mode, future_spec, source_ref):
         CompiledPythonModule.__init__(
             self,
             name         = "__main__",
             package_name = None,
             mode         = mode,
+            future_spec  = future_spec,
             source_ref   = source_ref
         )
 
@@ -587,10 +586,14 @@ class PythonMainModule(CompiledPythonModule):
 
     @classmethod
     def fromXML(cls, provider, source_ref, **args):
+        if "code_flags" in args:
+            future_spec = fromFlags(args["code_flags"])
+
         result = cls(
-            main_added = args["main_added"] == "True",
-            mode       = args["mode"],
-            source_ref = source_ref
+            main_added  = args["main_added"] == "True",
+            mode        = args["mode"],
+            future_spec = future_spec,
+            source_ref  = source_ref
         )
 
         from nuitka.ModuleRegistry import addRootModule
@@ -666,11 +669,10 @@ class PythonInternalModule(CompiledPythonModule):
             source_ref   = SourceCodeReference.fromFilenameAndLine(
                 filename = "internal",
                 line     = 0
-            )
+            ),
+            future_spec  = FutureSpec()
         )
 
-        self.future_spec = FutureSpec()
-
     @staticmethod
     def isInternalModule():
         return True
