@@ -23,16 +23,18 @@ source code comments with developer manual sections.
 """
 
 from nuitka.nodes.AssignNodes import (
-    ExpressionTargetTempVariableRef,
-    ExpressionTargetVariableRef,
     StatementAssignmentVariable,
+    StatementAssignmentVariableName,
     StatementReleaseVariable
 )
 from nuitka.nodes.AttributeNodes import (
     ExpressionAttributeLookup,
     ExpressionBuiltinHasattr
 )
-from nuitka.nodes.BuiltinRefNodes import ExpressionBuiltinRef
+from nuitka.nodes.BuiltinRefNodes import (
+    ExpressionBuiltinAnonymousRef,
+    makeExpressionBuiltinRef
+)
 from nuitka.nodes.CallNodes import ExpressionCall, ExpressionCallNoKeywords
 from nuitka.nodes.ClassNodes import (
     ExpressionClassBody,
@@ -57,14 +59,17 @@ from nuitka.nodes.FunctionNodes import (
     ExpressionFunctionRef
 )
 from nuitka.nodes.GlobalsLocalsNodes import (
-    ExpressionBuiltinLocals,
+    ExpressionBuiltinLocalsUpdated,
     StatementSetLocals
 )
+from nuitka.nodes.OutlineNodes import ExpressionOutlineBody
 from nuitka.nodes.ReturnNodes import StatementReturn
 from nuitka.nodes.SubscriptNodes import ExpressionSubscriptLookup
+from nuitka.nodes.TryNodes import StatementTry
 from nuitka.nodes.TypeNodes import ExpressionBuiltinType1
 from nuitka.nodes.VariableRefNodes import (
     ExpressionTempVariableRef,
+    ExpressionVariableNameRef,
     ExpressionVariableRef
 )
 from nuitka.PythonVersions import python_version
@@ -126,17 +131,13 @@ def _buildClassNode3(provider, node, source_ref):
         "__class__"
     )
 
-    class_target_variable_ref = ExpressionTargetVariableRef(
-        variable_name = "__class__",
-        variable      = class_variable,
-        source_ref    = source_ref
-    )
     class_variable_ref = ExpressionVariableRef(
-        variable_name = "__class__",
-        variable      = class_variable,
-        source_ref    = source_ref
+        variable   = class_variable,
+        source_ref = source_ref
     )
 
+    parent_module = provider.getParentModule()
+
     code_object = CodeObjectSpec(
         co_name           = node.name,
         co_kind           = "Class",
@@ -144,7 +145,10 @@ def _buildClassNode3(provider, node, source_ref):
         co_argcount       = 0,
         co_kwonlyargcount = 0,
         co_has_starlist   = False,
-        co_has_stardict   = False
+        co_has_stardict   = False,
+        co_filename       = parent_module.getRunTimeFilename(),
+        co_lineno         = source_ref.getLineNumber(),
+        future_spec       = parent_module.getFutureSpec()
     )
 
     body = buildFrameNode(
@@ -160,10 +164,6 @@ def _buildClassNode3(provider, node, source_ref):
         # The frame guard has nothing to tell its line number to.
         body.source_ref = source_ref
 
-    module_variable = class_creation_function.getVariableForAssignment(
-        "__module__"
-    )
-
     statements = [
         StatementSetLocals(
             new_locals = ExpressionTempVariableRef(
@@ -172,39 +172,27 @@ def _buildClassNode3(provider, node, source_ref):
             ),
             source_ref = source_ref
         ),
-        StatementAssignmentVariable(
-            variable_ref = ExpressionTargetVariableRef(
-                variable_name = "__module__",
-                variable      = module_variable,
-                source_ref    = source_ref
-            ),
-            source       = makeConstantRefNode(
+        StatementAssignmentVariableName(
+            variable_name = "__module__",
+            source        = makeConstantRefNode(
                 constant      = provider.getParentModule().getFullName(),
                 source_ref    = source_ref,
                 user_provided = True
             ),
-            source_ref   = source_ref
+            source_ref    = source_ref
         )
     ]
 
     if class_doc is not None:
-        doc_variable = class_creation_function.getVariableForAssignment(
-            "__doc__"
-        )
-
         statements.append(
-            StatementAssignmentVariable(
-                variable_ref = ExpressionTargetVariableRef(
-                    variable_name = "__doc__",
-                    variable      = doc_variable,
-                    source_ref    = source_ref
-                ),
-                source       = makeConstantRefNode(
+            StatementAssignmentVariableName(
+                variable_name = "__doc__",
+                source        = makeConstantRefNode(
                     constant      = class_doc,
                     source_ref    = source_ref,
                     user_provided = True
                 ),
-                source_ref   = source_ref
+                source_ref    = source_ref
             )
         )
 
@@ -228,14 +216,10 @@ def _buildClassNode3(provider, node, source_ref):
             )
 
         statements.append(
-            StatementAssignmentVariable(
-                variable_ref = ExpressionTargetVariableRef(
-                    variable_name = "__qualname__",
-                    variable      = qualname_variable,
-                    source_ref    = source_ref
-                ),
-                source       = qualname_ref,
-                source_ref   = source_ref
+            StatementAssignmentVariableName(
+                variable_name = qualname_variable.getName(),
+                source        = qualname_ref,
+                source_ref    = source_ref
             )
         )
 
@@ -244,32 +228,24 @@ def _buildClassNode3(provider, node, source_ref):
 
     if python_version >= 360 and \
        class_creation_function.needsAnnotationsDictionary():
-        annotations_variable = class_creation_function.getVariableForAssignment(
-            "__annotations__"
-        )
-
         statements.append(
-            StatementAssignmentVariable(
-                variable_ref = ExpressionTargetVariableRef(
-                    variable_name = "__annotations__",
-                    variable      = annotations_variable,
-                    source_ref    = source_ref
-                ),
-                source       = makeConstantRefNode(
+            StatementAssignmentVariableName(
+                variable_name = "__annotations__",
+                source        = makeConstantRefNode(
                     constant      = {},
                     source_ref    = source_ref,
                     user_provided = True
                 ),
-                source_ref   = source_ref
+                source_ref    = source_ref
             )
         )
 
     statements.append(body)
 
     statements += [
-        StatementAssignmentVariable(
-            variable_ref = class_target_variable_ref,
-            source       = ExpressionCall(
+        StatementAssignmentVariableName(
+            variable_name = "__class__",
+            source        = ExpressionCall(
                 called     = ExpressionTempVariableRef(
                     variable   = tmp_metaclass,
                     source_ref = source_ref
@@ -286,7 +262,7 @@ def _buildClassNode3(provider, node, source_ref):
                             variable   = tmp_bases,
                             source_ref = source_ref
                         ),
-                        ExpressionBuiltinLocals(
+                        ExpressionBuiltinLocalsUpdated(
                             source_ref = source_ref
                         )
                     ),
@@ -298,7 +274,7 @@ def _buildClassNode3(provider, node, source_ref):
                 ),
                 source_ref = source_ref
             ),
-            source_ref   = source_ref
+            source_ref    = source_ref
         ),
         StatementReturn(
             expression = class_variable_ref,
@@ -360,25 +336,21 @@ def _buildClassNode3(provider, node, source_ref):
 
     statements = (
         StatementAssignmentVariable(
-            variable_ref = ExpressionTargetTempVariableRef(
-                variable   = tmp_bases,
-                source_ref = source_ref
-            ),
-            source       = makeSequenceCreationOrConstant(
+            variable   = tmp_bases,
+            source     = makeSequenceCreationOrConstant(
                 sequence_kind = "tuple",
                 elements      = buildNodeList(
-                    provider, node.bases, source_ref
+                    provider,
+                    node.bases,
+                    source_ref
                 ),
                 source_ref    = source_ref
             ),
-            source_ref   = source_ref
+            source_ref = source_ref
         ),
         StatementAssignmentVariable(
-            variable_ref = ExpressionTargetTempVariableRef(
-                variable   = tmp_class_decl_dict,
-                source_ref = source_ref
-            ),
-            source       = makeDictCreationOrConstant(
+            variable   = tmp_class_decl_dict,
+            source     = makeDictCreationOrConstant(
                 keys       = [
                     makeConstantRefNode(
                         constant      = keyword.arg,
@@ -395,14 +367,11 @@ def _buildClassNode3(provider, node, source_ref):
                 ],
                 source_ref = source_ref
             ),
-            source_ref   = source_ref
+            source_ref = source_ref
         ),
         StatementAssignmentVariable(
-            variable_ref = ExpressionTargetTempVariableRef(
-                variable   = tmp_metaclass,
-                source_ref = source_ref
-            ),
-            source       = ExpressionSelectMetaclass(
+            variable   = tmp_metaclass,
+            source     = ExpressionSelectMetaclass(
                 metaclass  = ExpressionConditional(
                     condition      = ExpressionComparisonIn(
                         left       = makeConstantRefNode(
@@ -433,7 +402,7 @@ def _buildClassNode3(provider, node, source_ref):
                             variable   = tmp_bases,
                             source_ref = source_ref
                         ),
-                        expression_no  = ExpressionBuiltinRef(
+                        expression_no  = makeExpressionBuiltinRef(
                             builtin_name = "type",
                             source_ref   = source_ref
                         ),
@@ -462,7 +431,7 @@ def _buildClassNode3(provider, node, source_ref):
                 ),
                 source_ref = source_ref
             ),
-            source_ref   = source_ref_orig
+            source_ref = source_ref_orig
         ),
         StatementConditional(
             condition  = ExpressionComparisonIn(
@@ -495,13 +464,10 @@ def _buildClassNode3(provider, node, source_ref):
             source_ref = source_ref
         ),
         StatementAssignmentVariable(
-            variable_ref = ExpressionTargetTempVariableRef(
-                variable   = tmp_prepared,
-                source_ref = source_ref
-            ),
-            source       = ExpressionConditional(
-                condition      = ExpressionBuiltinHasattr( # pylint: disable=E1120,E1123
-                    object     = ExpressionTempVariableRef(
+            variable   = tmp_prepared,
+            source     = ExpressionConditional(
+                condition      = ExpressionBuiltinHasattr(
+                    object_arg = ExpressionTempVariableRef(
                         variable   = tmp_metaclass,
                         source_ref = source_ref
                     ),
@@ -548,22 +514,17 @@ def _buildClassNode3(provider, node, source_ref):
                 ),
                 source_ref     = source_ref
             ),
-            source_ref   = source_ref
+            source_ref = source_ref
         ),
         StatementAssignmentVariable(
-            variable_ref = ExpressionTargetVariableRef(
-                variable_name = node.name,
-                source_ref    = source_ref
-            ),
-            source       = decorated_body,
-            source_ref   = source_ref
+            variable   = provider.getVariableForAssignment(node.name),
+            source     = decorated_body,
+            source_ref = source_ref
         ),
     )
 
     if python_version >= 340:
-        class_assign = statements[-1]
-
-        class_creation_function.qualname_setup = class_assign, qualname_assign
+        class_creation_function.qualname_setup = node.name, qualname_assign
 
     final = (
         StatementReleaseVariable(
@@ -605,6 +566,8 @@ def _buildClassNode2(provider, node, source_ref):
         source_ref = source_ref
     )
 
+    parent_module = provider.getParentModule()
+
     code_object = CodeObjectSpec(
         co_name           = node.name,
         co_kind           = "Class",
@@ -612,7 +575,10 @@ def _buildClassNode2(provider, node, source_ref):
         co_argcount       = 0,
         co_kwonlyargcount = 0,
         co_has_starlist   = False,
-        co_has_stardict   = False
+        co_has_stardict   = False,
+        co_filename       = parent_module.getRunTimeFilename(),
+        co_lineno         = source_ref.getLineNumber(),
+        future_spec       = parent_module.getFutureSpec()
     )
 
     body = buildFrameNode(
@@ -629,40 +595,34 @@ def _buildClassNode2(provider, node, source_ref):
     # returns its locals and cannot have other return statements contained, and
     # starts out with a variables "__module__" and potentially "__doc__" set.
     statements = [
-        StatementAssignmentVariable(
-            variable_ref = ExpressionTargetVariableRef(
-                variable_name = "__module__",
-                source_ref    = source_ref
-            ),
-            source       = makeConstantRefNode(
+        StatementAssignmentVariableName(
+            variable_name = "__module__",
+            source        = makeConstantRefNode(
                 constant      = provider.getParentModule().getFullName(),
                 source_ref    = source_ref,
                 user_provided = True
             ),
-            source_ref   = source_ref.atInternal()
+            source_ref    = source_ref.atInternal()
         )
     ]
 
     if class_doc is not None:
         statements.append(
-            StatementAssignmentVariable(
-                variable_ref = ExpressionTargetVariableRef(
-                    variable_name = "__doc__",
-                    source_ref    = source_ref
-                ),
-                source       = makeConstantRefNode(
+            StatementAssignmentVariableName(
+                variable_name = "__doc__",
+                source        = makeConstantRefNode(
                     constant      = class_doc,
                     source_ref    = source_ref,
                     user_provided = True
                 ),
-                source_ref   = source_ref.atInternal()
+                source_ref    = source_ref.atInternal()
             )
         )
 
     statements += [
         body,
         StatementReturn(
-            expression = ExpressionBuiltinLocals(
+            expression = ExpressionBuiltinLocalsUpdated(
                 source_ref = source_ref
             ),
             source_ref = source_ref.atInternal()
@@ -687,27 +647,131 @@ def _buildClassNode2(provider, node, source_ref):
     tmp_metaclass = provider.allocateTempVariable(temp_scope, "metaclass")
     tmp_class = provider.allocateTempVariable(temp_scope, "class")
 
-    statements = [
-        StatementAssignmentVariable(
-            variable_ref = ExpressionTargetTempVariableRef(
-                variable   = tmp_bases,
+    select_metaclass = ExpressionOutlineBody(
+        provider   = provider,
+        name       = "select_metaclass",
+        body       = None,
+        source_ref = source_ref
+    )
+
+    if node.bases:
+        tmp_base = select_metaclass.allocateTempVariable(
+            temp_scope = None,
+            name       = "base"
+        )
+
+        statements = (
+            StatementAssignmentVariable(
+                variable   = tmp_base,
+                source     = ExpressionSubscriptLookup(
+                    subscribed = ExpressionTempVariableRef(
+                        variable   = tmp_bases,
+                        source_ref = source_ref
+                    ),
+                    subscript  = makeConstantRefNode(
+                        constant      = 0,
+                        source_ref    = source_ref,
+                        user_provided = True
+                    ),
+                    source_ref = source_ref,
+                ),
                 source_ref = source_ref
             ),
-            source       = makeSequenceCreationOrConstant(
+            makeTryFinallyStatement(
+                provider,
+                tried      =             StatementTry(
+                    tried            = makeStatementsSequenceFromStatement(
+                        statement = StatementReturn(
+                            expression = ExpressionAttributeLookup(
+                                source         = ExpressionTempVariableRef(
+                                    variable   = tmp_base,
+                                    source_ref = source_ref
+                                ),
+                                attribute_name = "__class__",
+                                source_ref     = source_ref
+                            ),
+                            source_ref = source_ref
+                        )
+                    ),
+                    except_handler   = makeStatementsSequenceFromStatement(
+                        statement = StatementReturn(
+                            expression = ExpressionBuiltinType1(
+                                value      = ExpressionTempVariableRef(
+                                    variable   = tmp_base,
+                                    source_ref = source_ref
+                                ),
+                                source_ref = source_ref
+                            ),
+                            source_ref = source_ref
+                        )
+                    ),
+                    break_handler    = None,
+                    continue_handler = None,
+                    return_handler   = None,
+                    source_ref       = source_ref
+                ),
+                final      = StatementReleaseVariable(
+                    variable   = tmp_base,
+                    source_ref = source_ref
+                ),
+                source_ref = source_ref,
+                public_exc = False
+            ),
+        )
+    else:
+        statements = (
+            StatementTry(
+                tried            = makeStatementsSequenceFromStatement(
+                    statement = StatementReturn(
+                        # TODO: Should avoid checking __builtins__ for this.
+                        expression = ExpressionVariableNameRef(
+                            variable_name = "__metaclass__",
+                            source_ref    = source_ref
+                        ),
+                        source_ref = source_ref
+                    )
+                ),
+                except_handler   = makeStatementsSequenceFromStatement(
+                    statement = StatementReturn(
+                        expression = ExpressionBuiltinAnonymousRef(
+                            builtin_name = "classobj",
+                            source_ref   = source_ref
+                        ),
+                        source_ref = source_ref
+                    )
+                ),
+                break_handler    = None,
+                continue_handler = None,
+                return_handler   = None,
+                source_ref       = source_ref
+            ),
+        )
+
+    select_metaclass.setBody(
+        makeStatementsSequence(
+            statements = statements,
+            allow_none = False,
+            source_ref = source_ref
+        )
+    )
+
+    statements = [
+        StatementAssignmentVariable(
+            variable   = tmp_bases,
+            source     = makeSequenceCreationOrConstant(
                 sequence_kind = "tuple",
                 elements      = buildNodeList(
-                    provider, node.bases, source_ref
+                    provider   = provider,
+                    nodes      = node.bases,
+                    source_ref = source_ref
                 ),
                 source_ref    = source_ref
             ),
-            source_ref   = source_ref
+            source_ref = source_ref
         ),
         StatementAssignmentVariable(
-            variable_ref = ExpressionTargetTempVariableRef(
-                variable   = tmp_class_dict,
-                source_ref = source_ref
-            ),
-            source       =   ExpressionFunctionCall(
+            variable   = tmp_class_dict,
+            source     =   ExpressionFunctionCall(
                 function   = ExpressionFunctionCreation(
                     function_ref = ExpressionFunctionRef(
                         function_body = function_body,
@@ -722,14 +786,11 @@ def _buildClassNode2(provider, node, source_ref):
                 values     = (),
                 source_ref = source_ref
             ),
-            source_ref   = source_ref
+            source_ref = source_ref
         ),
         StatementAssignmentVariable(
-            variable_ref = ExpressionTargetTempVariableRef(
-                variable   = tmp_metaclass,
-                source_ref = source_ref
-            ),
-            source       = ExpressionConditional(
+            variable   = tmp_metaclass,
+            source     = ExpressionConditional(
                 condition      =  ExpressionComparisonIn(
                     left       = makeConstantRefNode(
                         constant      = "__metaclass__",
@@ -754,24 +815,14 @@ def _buildClassNode2(provider, node, source_ref):
                     ),
                     source_ref = source_ref
                 ),
-                expression_no  = ExpressionSelectMetaclass(
-                    metaclass  = None,
-                    bases      = ExpressionTempVariableRef(
-                        variable   = tmp_bases,
-                        source_ref = source_ref
-                    ),
-                    source_ref = source_ref
-                ),
+                expression_no  = select_metaclass,
                 source_ref     = source_ref
             ),
-            source_ref   = source_ref
+            source_ref = source_ref
         ),
         StatementAssignmentVariable(
-            variable_ref = ExpressionTargetTempVariableRef(
-                variable   = tmp_class,
-                source_ref = source_ref
-            ),
-            source       = ExpressionCallNoKeywords(
+            variable   = tmp_class,
+            source     = ExpressionCallNoKeywords(
                 called     = ExpressionTempVariableRef(
                     variable   = tmp_metaclass,
                     source_ref = source_ref
@@ -796,7 +847,7 @@ def _buildClassNode2(provider, node, source_ref):
                 ),
                 source_ref = source_ref
             ),
-            source_ref   = source_ref
+            source_ref = source_ref
         ),
     ]
 
@@ -807,11 +858,8 @@ def _buildClassNode2(provider, node, source_ref):
         ):
         statements.append(
             StatementAssignmentVariable(
-                variable_ref = ExpressionTargetTempVariableRef(
-                    variable   = tmp_class,
-                    source_ref = source_ref
-                ),
-                source       = ExpressionCallNoKeywords(
+                variable   = tmp_class,
+                source     = ExpressionCallNoKeywords(
                     called     = decorator,
                     args       = ExpressionMakeTuple(
                         elements   = (
@@ -824,21 +872,18 @@ def _buildClassNode2(provider, node, source_ref):
                     ),
                     source_ref = decorator.getSourceReference()
                 ),
-                source_ref   = decorator.getSourceReference()
+                source_ref = decorator.getSourceReference()
             )
         )
 
     statements.append(
-        StatementAssignmentVariable(
-            variable_ref = ExpressionTargetVariableRef(
-                variable_name = node.name,
-                source_ref    = source_ref
-            ),
-            source       = ExpressionTempVariableRef(
+        StatementAssignmentVariableName(
+            variable_name = node.name,
+            source        = ExpressionTempVariableRef(
                 variable   = tmp_class,
                 source_ref = source_ref
             ),
-            source_ref   = source_ref
+            source_ref    = source_ref
         )
     )
 
