@@ -23,8 +23,6 @@ source code comments with developer manual sections.
 """
 
 from nuitka.nodes.AssignNodes import (
-    ExpressionTargetTempVariableRef,
-    ExpressionTempVariableRef,
     StatementAssignmentVariable,
     StatementReleaseVariable
 )
@@ -38,7 +36,10 @@ from nuitka.nodes.ContainerOperationNodes import (
     StatementSetOperationAdd
 )
 from nuitka.nodes.DictionaryNodes import StatementDictOperationSet
-from nuitka.nodes.FrameNodes import StatementsFrame
+from nuitka.nodes.FrameNodes import (
+    StatementsFrameFunction,
+    StatementsFrameGenerator
+)
 from nuitka.nodes.FunctionNodes import (
     ExpressionFunctionBody,
     ExpressionFunctionCall,
@@ -50,10 +51,7 @@ from nuitka.nodes.GeneratorNodes import (
     ExpressionMakeGeneratorObject
 )
 from nuitka.nodes.LoopNodes import StatementLoop, StatementLoopBreak
-from nuitka.nodes.NodeMakingHelpers import (
-    makeVariableRefNode,
-    makeVariableTargetRefNode
-)
+from nuitka.nodes.NodeMakingHelpers import makeVariableRefNode
 from nuitka.nodes.OutlineNodes import ExpressionOutlineBody
 from nuitka.nodes.ParameterSpecs import ParameterSpec
 from nuitka.nodes.ReturnNodes import StatementReturn
@@ -61,6 +59,7 @@ from nuitka.nodes.StatementNodes import (
     StatementExpressionOnly,
     StatementsSequence
 )
+from nuitka.nodes.VariableRefNodes import ExpressionTempVariableRef
 from nuitka.nodes.YieldNodes import ExpressionYield
 from nuitka.PythonVersions import python_version
 
@@ -69,6 +68,7 @@ from .Helpers import (
     buildNodeList,
     getKind,
     makeStatementsSequenceFromStatement,
+    makeStatementsSequenceFromStatements,
     mergeStatements
 )
 from .ReformulationAssignmentStatements import buildAssignmentStatements
@@ -126,23 +126,16 @@ def _buildPython2ListContraction(provider, node, source_ref):
         )
     )
 
-    statements = (
-        makeTryFinallyStatement(
-            provider   = function_body,
-            tried      = statements,
-            final      = release_statements,
-            source_ref = source_ref.atInternal()
-        ),
+    statement = makeTryFinallyStatement(
+        provider   = function_body,
+        tried      = statements,
+        final      = release_statements,
+        source_ref = source_ref.atInternal()
     )
 
     function_body.setBody(
         makeStatementsSequenceFromStatement(
-            statement = StatementsFrame(
-                statements  = mergeStatements(statements, False),
-                guard_mode  = "pass_through",
-                code_object = None,
-                source_ref  = source_ref
-            )
+            statement = statement
         )
     )
 
@@ -205,15 +198,109 @@ def buildGeneratorExpressionNode(provider, node, source_ref):
 
     assert getKind(node) == "GeneratorExp"
 
-    return _buildContractionNode(
-        provider    = provider,
-        node        = node,
-        name        = "<genexpr>",
-        emit_class  = ExpressionYield,
-        start_value = None,
-        source_ref  = source_ref
+    function_body = ExpressionOutlineBody(
+        provider   = provider,
+        name       = "genexpr",
+        source_ref = source_ref
     )
 
+    iter_tmp = function_body.allocateTempVariable(
+        temp_scope = None,
+        name       = ".0"
+    )
+
+    parent_module = provider.getParentModule()
+
+    code_object = CodeObjectSpec(
+        co_name           = "<genexpr>",
+        co_kind           = "Generator",
+        co_varnames       = (".0",),
+        co_argcount       = 1,
+        co_kwonlyargcount = 0,
+        co_has_starlist   = False,
+        co_has_stardict   = False,
+        co_filename       = parent_module.getRunTimeFilename(),
+        co_lineno         = source_ref.getLineNumber(),
+        future_spec       = parent_module.getFutureSpec()
+    )
+
+    code_body = ExpressionGeneratorObjectBody(
+        provider   = provider,
+        name       = "<genexpr>",
+        flags      = set(),
+        source_ref = source_ref
+    )
+
+    function_body.setBody(
+        makeStatementsSequenceFromStatements(
+            StatementAssignmentVariable(
+                variable   = iter_tmp,
+                source     = ExpressionBuiltinIter1(
+                    value      = buildNode(
+                        provider   = provider,
+                        node       = node.generators[0].iter,
+                        source_ref = source_ref
+                    ),
+                    source_ref = source_ref
+                ),
+                source_ref = source_ref
+            ),
+            makeTryFinallyStatement(
+                provider   = function_body,
+                tried      = StatementReturn(
+                    expression = ExpressionMakeGeneratorObject(
+                        generator_ref = ExpressionFunctionRef(
+                            function_body = code_body,
+                            source_ref    = source_ref
+                        ),
+                        code_object   = code_object,
+                        source_ref    = source_ref
+                    ),
+                    source_ref = source_ref
+                ),
+                final      = StatementReleaseVariable(
+                    variable   = iter_tmp,
+                    source_ref = source_ref
+                ),
+                source_ref = source_ref
+            )
+        )
+    )
+
+    statements, release_statements = _buildContractionBodyNode(
+        function_body   = code_body,
+        provider        = provider,
+        node            = node,
+        emit_class      = ExpressionYield,
+        iter_tmp        = iter_tmp,
+        temp_scope      = None,
+        start_value     = None,
+        container_tmp   = None,
+        assign_provider = False,
+        source_ref      = source_ref,
+    )
+
+    statements = (
+        makeTryFinallyStatement(
+            provider   = function_body,
+            tried      = statements,
+            final      = release_statements,
+            source_ref = source_ref.atInternal()
+        ),
+    )
+
+    code_body.setBody(
+        makeStatementsSequenceFromStatement(
+            statement = StatementsFrameGenerator(
+                statements  = mergeStatements(statements, False),
+                code_object = code_object,
+                source_ref  = source_ref
+            )
+        )
+    )
+
+    return function_body
+
 
 def _buildContractionBodyNode(provider, node, emit_class, start_value,
                               container_tmp, iter_tmp, temp_scope,
@@ -238,11 +325,8 @@ def _buildContractionBodyNode(provider, node, emit_class, start_value,
     if assign_provider:
         statements = [
             StatementAssignmentVariable(
-                variable_ref = makeVariableTargetRefNode(
-                    variable   = iter_tmp,
-                    source_ref = source_ref
-                ),
-                source       = ExpressionBuiltinIter1(
+                variable   = iter_tmp,
+                source     = ExpressionBuiltinIter1(
                     value      = buildNode(
                         provider   = provider,
                         node       = node.generators[0].iter,
@@ -250,7 +334,7 @@ def _buildContractionBodyNode(provider, node, emit_class, start_value,
                     ),
                     source_ref = source_ref
                 ),
-                source_ref   = source_ref.atInternal()
+                source_ref = source_ref.atInternal()
             )
         ]
     else:
@@ -259,12 +343,9 @@ def _buildContractionBodyNode(provider, node, emit_class, start_value,
     if start_value is not None:
         statements.append(
             StatementAssignmentVariable(
-                variable_ref = ExpressionTargetTempVariableRef(
-                    variable   = container_tmp,
-                    source_ref = source_ref
-                ),
-                source       = start_value,
-                source_ref   = source_ref.atInternal()
+                variable   = container_tmp,
+                source     = start_value,
+                source_ref = source_ref.atInternal()
             )
         )
 
@@ -362,12 +443,9 @@ def _buildContractionBodyNode(provider, node, emit_class, start_value,
 
             nested_statements = [
                 StatementAssignmentVariable(
-                    variable_ref = ExpressionTargetTempVariableRef(
-                        variable   = tmp_iter_variable,
-                        source_ref = source_ref
-                    ),
-                    source       = value_iterator,
-                    source_ref   = source_ref
+                    variable   = tmp_iter_variable,
+                    source     = value_iterator,
+                    source_ref = source_ref
                 )
             ]
 
@@ -379,15 +457,12 @@ def _buildContractionBodyNode(provider, node, emit_class, start_value,
         loop_statements = [
             makeTryExceptSingleHandlerNode(
                 tried          = StatementAssignmentVariable(
-                    variable_ref = ExpressionTargetTempVariableRef(
-                        variable   = tmp_value_variable,
-                        source_ref = source_ref
-                    ),
-                    source       = ExpressionBuiltinNext1(
+                    variable   = tmp_value_variable,
+                    source     = ExpressionBuiltinNext1(
                         value      = iterator_ref,
                         source_ref = source_ref
                     ),
-                    source_ref   = source_ref
+                    source_ref = source_ref
                 ),
                 exception_name = "StopIteration",
                 handler_body   = StatementLoopBreak(
@@ -474,7 +549,7 @@ def _buildContractionNode(provider, node, name, emit_class, start_value,
     # described in the developer manual. They use a lot of temporary names,
     # nested blocks, etc. and so a lot of variable names.
 
-    # TODO: No function ought to be necessary.
+    # TODO: No function ought to be necessary, only a variable scope is.
     function_body = ExpressionFunctionBody(
         provider   = provider,
         name       = name,
@@ -496,57 +571,28 @@ def _buildContractionNode(provider, node, name, emit_class, start_value,
     )
     assert iter_tmp.isParameterVariable()
 
+    parent_module = provider.getParentModule()
+
     code_object = CodeObjectSpec(
         co_name           = name,
-        co_kind           = "Generator" if emit_class is ExpressionYield else "Function",
+        co_kind           = "Function",
         co_varnames       = function_body.getParameters().getParameterNames(),
         co_argcount       = len(function_body.getParameters().getParameterNames()),
         co_kwonlyargcount = 0,
         co_has_starlist   = False,
         co_has_stardict   = False,
+        co_filename       = parent_module.getRunTimeFilename(),
+        co_lineno         = source_ref.getLineNumber(),
+        future_spec       = parent_module.getFutureSpec()
     )
 
-    if emit_class is ExpressionYield:
-        code_body = ExpressionGeneratorObjectBody(
-            provider   = function_body,
-            name       = "<genexpr>",
-            flags      = set(),
-            source_ref = source_ref
-        )
-
-        iter_tmp = code_body.getVariableForReference(
-            variable_name = ".0"
-        )
-        assert iter_tmp.isLocalVariable()
-
-        function_body.setBody(
-            makeStatementsSequenceFromStatement(
-                statement = StatementReturn(
-                    expression = ExpressionMakeGeneratorObject(
-                        generator_ref = ExpressionFunctionRef(
-                            function_body = code_body,
-                            source_ref    = source_ref
-                        ),
-                        code_object   = code_object,
-                        source_ref    = source_ref
-                    ),
-                    source_ref = source_ref
-                )
-            )
-        )
-    else:
-        code_body = function_body
-
-    if start_value is not None:
-        container_tmp = code_body.allocateTempVariable(
-            temp_scope = None,
-            name       = "contraction_result"
-        )
-    else:
-        container_tmp = None
+    container_tmp = function_body.allocateTempVariable(
+        temp_scope = None,
+        name       = "contraction_result"
+    )
 
     statements, release_statements = _buildContractionBodyNode(
-        function_body   = code_body,
+        function_body   = function_body,
         provider        = provider,
         node            = node,
         emit_class      = emit_class,
@@ -558,16 +604,17 @@ def _buildContractionNode(provider, node, name, emit_class, start_value,
         source_ref      = source_ref,
     )
 
-    if start_value is not None:
-        statements.append(
-            StatementReturn(
-                expression = ExpressionTempVariableRef(
-                    variable   = container_tmp,
-                    source_ref = source_ref
-                ),
+    assert start_value
+
+    statements.append(
+        StatementReturn(
+            expression = ExpressionTempVariableRef(
+                variable   = container_tmp,
                 source_ref = source_ref
-            )
+            ),
+            source_ref = source_ref
         )
+    )
 
     statements = (
         makeTryFinallyStatement(
@@ -578,16 +625,10 @@ def _buildContractionNode(provider, node, name, emit_class, start_value,
         ),
     )
 
-    if emit_class is ExpressionYield:
-        guard_mode = "generator"
-    else:
-        guard_mode = "full"
-
-    code_body.setBody(
+    function_body.setBody(
         makeStatementsSequenceFromStatement(
-            statement = StatementsFrame(
+            statement = StatementsFrameFunction(
                 statements  = mergeStatements(statements, False),
-                guard_mode  = guard_mode,
                 code_object = code_object,
                 source_ref  = source_ref
             )
